# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import io
import unittest
from unittest.mock import patch
from Xhpc.email import *
from Xhpc.io_utils import get_first_line

import pkg_resources
test_folder = pkg_resources.resource_filename("Xhpc", "test")


class TestEmail(unittest.TestCase):

    def setUp(self) -> None:

        self.config_folder = '%s/configs' % test_folder
        os.makedirs(self.config_folder)

        self.path1 = '%s/file1.txt' % self.config_folder
        self.path2 = '%s/file2.txt' % self.config_folder
        with open(self.path1, 'w') as o1, open(self.path2, 'w') as o2:
            o1.write('line1\nline2\nline3\n')
            o2.write('\n')
        self.valid_emails = ['this@is.ok', 'this.is@ok.too', 'and@this.ok.too']
        self.invalid_emails = ['wrong@', 'not.ok', '@try.again']

        self.path_out = '%s/created_config.txt' % self.config_folder

        self.email_fp1 = '%s/email_fp1.txt' % self.config_folder
        self.email_fp2 = '%s/email_fp2.txt' % self.config_folder
        self.email_fp3 = '%s/email_fp3.txt' % self.config_folder
        with open(self.email_fp1, 'w') as o1, open(self.email_fp2, 'w') as o2:
            o1.write('this@is.ok\n')
            o2.write('wrong@\n')

    def test_validate_email(self):
        for valid_email in self.valid_emails:
            self.assertTrue(validate_email(valid_email))
        for invalid_email in self.invalid_emails:
            self.assertFalse(validate_email(invalid_email))

    @patch('builtins.input', return_value='this@is.ok')
    def test_valid_get_email(self, mock_input):
        self.assertEqual(get_email(), 'this@is.ok')

    @patch('builtins.input', side_effect=['wrong@', 'not.ok', '@not.ok'])
    def test_invalid_get_email(self, mock_input):
        with self.assertRaises(ValueError) as cm:
            get_email()
        self.assertEqual(str(cm.exception), 'wrong@ not a valid email address')
        with self.assertRaises(ValueError) as cm:
            get_email()
        self.assertEqual(str(cm.exception), 'not.ok not a valid email address')
        with self.assertRaises(ValueError) as cm:
            get_email()
        self.assertEqual(str(cm.exception), '@not.ok not a valid email address')

    @patch('builtins.input', return_value='this@is.ok')
    def test_create_valid_config(self, input):
        create_config(self.path_out)
        first_line = get_first_line(self.path_out)
        self.assertEqual(first_line, 'this@is.ok')

    @patch('builtins.input', return_value='@not.ok')
    def test_create_invalid_config(self, input):
        with self.assertRaises(ValueError) as cm:
            create_config(self.path_out)
        self.assertEqual(str(cm.exception), '@not.ok not a valid email address')

    def test_edit_config(self):
        email = edit_config(self.path1, False)
        self.assertEqual(email, 'line1')

    @patch('builtins.input', return_value='this@is.ok')
    def test_edit_valid_config(self, input):
        email = edit_config(self.path1, True)
        self.assertEqual(email, 'this@is.ok')

    def test_write_email(self):
        with patch('sys.stdout', new=io.StringIO()) as fake_out:
            write_email(self.path_out, 'sometext')
            self.assertEqual('Written: %s\n' % self.path_out,
                             fake_out.getvalue())
            with open(self.path_out) as f:
                for line in f:
                    self.assertEqual(line.strip(), 'sometext')

    def test_get_email_address(self):
        args = {'config_email': False}
        get_email_address(args, self.path1)
        self.assertEqual(args['email_address'], 'line1')
        get_email_address(args, self.email_fp1)
        self.assertEqual(args['email_address'], 'this@is.ok')

    @patch('builtins.input', return_value='this@is.ok')
    def test_get_email_address_create(self, input):
        args = {}
        get_email_address(args, self.email_fp3)
        self.assertEqual(args['email_address'], 'this@is.ok')

    @patch('builtins.input', return_value='this@is.ok')
    def test_get_email_address_query(self, input):
        args = {'config_email': True}
        get_email_address(args, self.path1)
        self.assertEqual(args['email_address'], 'this@is.ok')
        args = {'config_email': True}
        get_email_address(args, self.email_fp2)
        self.assertEqual(args['email_address'], 'this@is.ok')

    def tearDown(self) -> None:
        for path in [
            self.path1,
            self.path2,
            self.path_out,
            self.email_fp1,
            self.email_fp2,
            self.email_fp3
        ]:
            if os.path.isfile(path):
                os.remove(path)
        for folder in [self.config_folder]:
            os.rmdir(folder)


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import itertools
import os
from os.path import dirname, isdir, isfile


def print_scratch_message(args, scratch_path):
    """

    Parameters
    ----------
    args : dict
        All arguments, including:
            quiet : bool
                Do not print anything
    scratch_path : str
        Either of "scratch", "userscratch" or "localscratch"

    Returns
    -------
    message : str
        Message about scratch
    """
    message = ''
    if not args['quiet']:
        message = 'Creating scratch location "%s" ' % scratch_path
        if args['move']:
            message += "and moving files to/from for computing."
        else:
            message += 'but not using it (need to set "--move" too)'
    return message


def get_scratch_path(args: dict) -> str:
    """Get the path to the scratch folder to use to move and compute.
    If the use activate more than one of the three scratch arguments, then the
    scratch path to be used is in order of top to bottom priority, i.e.:
    - localscratch (fastest file system because on the running node)
    - scratch  (automatically created for the job, and then deleted)
    - userscratch (always exists and thus can be looked at, but not backed-up)

    Parameters
    ----------
    args : dict
        All arguments, including:
            scratch: str
                Path to scratch folder or None if not set
            userscratch: str
                Path to user scratch folder or None if not set
            localscratch: tuple
                Path "/localscratch" or command None (default) if not set

    Returns
    -------
    scratch_val : str
        Value for the scratch key folder
    """
    for scratch_path in ['localscratch', 'scratch', 'userscratch']:
        if args[scratch_path]:
            print_scratch_message(args, scratch_path)
            scratch_val = args[scratch_path]
            return scratch_val


def get_scratching_commands(args: dict) -> None:
    """Get the scratch folder creation and deletion commands.

    Parameters
    ----------
    args : dict
        All arguments, including:
            scratch: str
                Path to scratch folder or None if not set
            userscratch: str
                Path to user scratch folder or None if not set
            localscratch: tuple
                Path "/localscratch" or command None (default) if not set
            torque: bool
                Adapt to Torque
    """
    # Define scratch directory
    scratch_path = '%s/${%s}' % (get_scratch_path(args), args['job_id'])
    # Get commands to create and more to scratch directory
    args['scratching'].append('# Define and create a scratch directory')
    args['scratching'].append('SCRATCH_FOLDER="%s"' % scratch_path)
    args['scratching'].append('mkdir -p ${SCRATCH_FOLDER}')
    args['scratching'].append('cd ${SCRATCH_FOLDER}')
    args['scratching'].append('echo Working directory is ${SCRATCH_FOLDER}')


def get_in_out(paths: set) -> dict:
    """Get the paths that would need to be move to/from scratch locations.

    Parameters
    ----------
    paths : set
        Paths to file or folders to move when using scratch folder

    Returns
    -------
    in_out : dict
        Sets of existing files and folder that will be moved in and
        of non-existing paths that will be move back.
    """
    in_out = {'files': set(), 'folders': set(), 'out': set()}
    for path in paths:
        if isfile(path):
            in_out['files'].add(path)
        elif isdir(path):
            in_out['folders'].add(path)
        elif isdir(dirname(path)):
            in_out['folders'].add(dirname(path))
        else:
            in_out['out'].add(path)
    return in_out


def get_min_files(files: set, min_folders: set) -> set:
    """Reduce the set of files to those not already within the folders.

    Parameters
    ----------
    files : set
        Sets of existing files that will be moved in scratch
    min_folders : set
        Keep the most basal of the existing folders passed in command

    Returns
    -------
    min_files : set
        Files not present with the minimum set of folders
    """
    min_files = set()
    for f1 in files:
        for f2 in min_folders:
            if f2 in dirname(f1):
                break
        else:
            min_files.add(f1)
    return min_files


def get_min_folders(folders: set, included: set) -> set:
    """Reduce the set of folder to that which is most basal so that

    Parameters
    ----------
    folders : set
        Sets of existing folder that will be moved in scratch
    included : set
        Paths to the folders that the user passed to option `--p-include`

    Returns
    -------
    min_folders : set
        Minimum set of existing folders to contain all folders passed in command
    """
    # Get the folders entailed within others (so that these need not be moved
    # specifically, they will be moved as being within these "others")
    min_folders, exclude = set(), set()
    for f1, f2 in itertools.combinations(sorted(folders), 2):
        if f1 != f2 and f1 in f2:
            exclude.add(f2)

    # Expand the collection of folders with those entailed within the
    # folders passed to option `--p-include` (same reason)
    for f1 in folders:
        for f2 in included:
            if f2 in f1:
                exclude.add(f1)

    # Actual reduction to remove all folders contained within a broader path
    min_folders = folders.difference(exclude) | included
    return min_folders


def get_include_commands(args: dict) -> set:
    """Get the "--include" option for the rsync command to exclude files
    and/or folder provided b the user.

    Parameters
    ----------
    args : dict
        All arguments, including:
            include : tuple
                Folder to not move to and from scratch using rsync (must exist)

    Returns
    -------
    included : set
        Paths to the folders that the user passed to option `--p-include`
    """
    included = set()
    if args['include']:
        for folder_ in args['include']:
            if not isdir(folder_):
                continue
            folder = os.path.abspath(folder_)
            included.add(folder)
            scratch = '${SCRATCH_FOLDER}%s' % folder
            args['mkdir'].add('mkdir -p %s' % dirname(scratch))
            args['move_to'].add('rsync -aqru %s/ %s' % (folder, scratch))
            args['move_from'].add('rsync -aqru %s/ %s' % (scratch, folder))
    return included


def move_to(args: dict, path: str, is_folder: bool) -> None:
    """

    Parameters
    ----------
    args : dict
        All arguments.
    path : str
        A folder or a file to move to scratch
    is_folder : bool
        Whether the path is a folder (True) or a file (False)
    """
    local = path
    remote_from = '${SCRATCH_FOLDER}%s' % path
    if is_folder:
        local += '/'
        remote_from = '${SCRATCH_FOLDER}%s/' % path
    remote_to = '${SCRATCH_FOLDER}%s' % path
    if path not in set(args['exclude']):
        args['mkdir'].add('mkdir -p %s' % dirname(remote_to))
        args['move_to'].add('rsync -aqru %s %s' % (local, remote_to))
        args['move_from'].add('rsync -aqru %s %s' % (remote_from, path))


def get_min_paths(in_out: dict, included: set) -> dict:
    """Get the minimum set of folders and files to move to scratch.

    Parameters
    ----------
    in_out : dict
        Sets of existing files and folder that will be moved in and
        of non-existing paths that will be move back.
    included : set
        Paths to the folders that the user passed to option `--p-include`

    Returns
    -------
    min_paths : dict
        Minimum set of existing folders to contain all folders passed in
        command (key "folders") and files not present with the minimum set of
        folders (key "files")
    """
    min_folders = get_min_folders(in_out['folders'], included)
    min_files = get_min_files(in_out['files'], min_folders)
    min_paths = {'folders': min_folders, 'files': min_files}
    return min_paths


def get_in_commands(args: dict, min_paths: dict) -> None:
    """Get command that move the inputs to scratch.

    Parameters
    ----------
    args : dict
        All arguments.
    min_paths : set
        Folder and files that must be move to the scratch folder
    """
    for min_folder in min_paths['folders']:
        move_to(args, min_folder, True)
    for min_file in min_paths['files']:
        move_to(args, min_file, False)


def get_out_commands(args: dict, in_out: dict) -> None:
    """Get command that move the inputs in and outputs out.

    Parameters
    ----------
    args : dict
        All arguments.
    in_out : dict
        Sets of existing files and folder that will be moved in and
        of non-existing paths that will be move back.
    """
    for path in in_out['out']:
        source = '${SCRATCH_FOLDER}%s' % path
        args['move_from'].update([
            'if [ -d %s ]; then mkdir -p %s; rsync -aqru %s/ %s; fi' % (
                source, path, source, path),
            'if [ -f %s ]; then rsync -aqru %s %s; fi' % (
                source, source, path)])


def get_relocating_commands(args: dict) -> None:
    """Get command that move the inputs in and outputs out.

    Parameters
    ----------
    args : dict
        All arguments, including:
            paths : set
                Paths to file or folders to move when using scratch folder
    """
    # Folders to move to and from scratch
    included = get_include_commands(args)
    # Get paths to existing folders and files, and non-existing ones
    in_out = get_in_out(args['paths'])
    min_paths = get_min_paths(in_out, included)
    # Move in to scratch
    get_in_commands(args, min_paths)
    # Move out from scratch
    get_out_commands(args, in_out)


def get_clearing_commands(args: dict):
    """Collect all the commands that clear the scratch location.

    Parameters
    ----------
    args : dict
        All arguments.
            clear_scratch : bool
                Whether to cleat the scratch area at the end of the job or not
            torque: bool
                Adapt to Torque
    """
    if args['clear_scratch']:
        # Get commands to move away and delete scratch directory
        if args['torque']:
            args['clear'].append('cd ${PBS_O_WORKDIR}')
            args['clear'].append('rm -rf ${SCRATCH_FOLDER}')
        else:
            args['clear'].append('cd ${SLURM_SUBMIT_DIR}')
            args['clear'].append('rm -rf ${SCRATCH_FOLDER}')


def go_to_work(args: dict) -> None:
    """Get the working folder to go to and echo.

    Parameters
    ----------
    args : dict
        All arguments, including:
            torque: bool
                Adapt to Torque
    """
    args['move_to'] = ['# Move to the working directory and say it']
    work_dir = 'SLURM_SUBMIT_DIR'
    if args['torque']:
        work_dir = 'PBS_O_WORKDIR'
    args['move_to'].append('cd $%s' % work_dir)
    args['move_to'].append('echo Working directory is $%s' % work_dir)


def get_relocation(args: dict) -> None:
    """Collect all the relocation commands that move files and folders
    to/from scratch folder.
    This results in extending the `args` dictionary with the "in", "out" and
    "mv" keys that point to lists of command.

    Parameters
    ----------
    args : dict
        All arguments, including:
            move : bool
                Move files/folders to chosen scratch location
    """
    if args['scratch'] or args['userscratch'] or args['localscratch']:
        # Get scratch folder creation and deletion commands
        get_scratching_commands(args)
        if args['move']:
            # Get command that move the inputs in and outputs out
            get_relocating_commands(args)
        else:
            go_to_work(args)
        # Get command that clear the scratch location
        get_clearing_commands(args)
    else:
        # Switch to working directory; default is home directory
        go_to_work(args)



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import sys
from os.path import abspath
from Xhpc.nodes import get_nodelist, allocate_nodes, get_nodes_ppn


def set_account(args: dict) -> str:
    """Get the directives for the user account.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            account : str
                Name of the account
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['account']:
        if args['torque']:
            directive = '#PBS -A %s' % args['account']
        else:
            directive = '#SBATCH --account=%s' % args['account']
        return directive


def set_partition(args: dict) -> str:
    """Get the directives for the allocation of a partition based on user input.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            partition: str
                Partition name
            gpu: bool
                Query a gpu (experimental)
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        directive = '#PBS -q %s' % args['partition']
    else:
        partition = '#SBATCH --partition=normal'
        if args['gpu']:
            partition = '#SBATCH --partition=accel'
        elif args['partition']:
            partition = '#SBATCH --partition=%s' % args['partition']
        directive = partition
    return directive


def set_environment(args: dict) -> str:
    """Get the directives for the user environment.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        directive = '#PBS -V'
    else:
        directive = '#SBATCH --export=ALL'
    return directive


def set_job(args: dict) -> str:
    """Get the directives for the user environment.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            job: str
                Job name
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        directive = '#PBS -N %s' % args['job']
    else:
        directive = '#SBATCH --job-name=%s' % args['job']
    return directive


def set_email(args: dict) -> str:
    """Get the directives for the emailing.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            email: bool
                Send email (always if fail)
            torque: bool
                Adapt to Torque
            email_address : str
                email address of the user

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        if args['email']:
            directive = '#PBS -m ae'
        else:
            directive = '#PBS -m a'
        directive += '\n#PBS -M %s' % args['email_address']
    else:
        if args['email']:
            directive = '#SBATCH --mail-type=END,FAIL,TIME_LIMIT_80'
        else:
            directive = '#SBATCH --mail-type=FAIL,TIME_LIMIT_80'
        directive += '\n#SBATCH --mail-user=%s' % args['email_address']
    return directive


def set_stdout_stderr(args: dict) -> str:
    """Get the directives for the working directory where the job stdout and
    stderr files will be written. The path (no extension) of the stdout and
    stderr files will also be collected.
    This results in extending the `args` dictionary with the "std_path" key
    pointing to this path as a value. The environment variable pointing to
    the job ID is also registered under key "job_id".

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            workdir: str
                Working directory
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        work_dir = '${PBS_O_WORKDIR}/${PBS_JOBNAME}'
        if args['workdir']:
            work_dir = '%s/${PBS_JOBNAME}' % abspath(args['workdir'])
        job_id = 'PBS_JOBID'
        std_path = 'localhost:%s_${%s}' % (work_dir, job_id)
        directive = '#PBS -o %s.o\n#PBS -e %s.e' % (std_path, std_path)
    else:
        work_dir = '${SLURM_SUBMIT_DIR}'
        if args['workdir']:
            work_dir = abspath(args['workdir'])
        job_id = 'SLURM_JOB_ID'
        std_path_ = 'slurm-%sx_%sj' % ('%', '%')
        std_path = 'slurm-${SLURM_JOB_NAME}_${SLURM_JOB_ID}'
        directive = '#SBATCH -o %s.o\n#SBATCH -e %s.e' % (std_path_, std_path_)
    args['std_path'] = std_path
    args['job_id'] = job_id
    return directive


def set_time(args: dict) -> str:
    """Get the directives for the time.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            time: str
                Wall time limit
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        directive = '#PBS -l walltime=%s:00:00' % args['time']
    else:
        directive = '#SBATCH --time=%s:00:00' % args['time']
    return directive


def set_memory(args: dict) -> str:
    """Get the directives for the memory.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            mem: tuple
                Requested memory
            mem_per_cpu: tuple
                Requested memory per cpu
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    for mem in ['mem_per_cpu', 'mem']:
        if not args[mem]:
            continue
        if args['torque']:
            directive = '#PBS -l mem=%s' % (''.join(args[mem])).lower()
        else:
            num, byt = args[mem]
            directive = '#SBATCH --%s=%s%s' % (
                mem.replace('_', '-'), num, byt.upper())
        return directive


def set_localscratch(args: dict) -> str:
    """Get the directives for the localscratch --gres option.
    Only available for the SAGA cluster and on Slurm.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            localscratch: tuple
                Use localscratch with the provided memory amount (in gb)
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if not args['torque'] and args['localscratch']:
        directive = '#SBATCH --gres=localscratch:%sG' % args['localscratch']
        args['localscratch'] = '/localscratch'
        return directive


def set_nodes(args: dict) -> str:
    """Get the directives for the nodes.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            nodes: tuple
                Node names
            torque: bool
                Adapt to Torque

    Returns
    -------
    directive : str
        Current directive
    """
    if args['nodes']:
        directive = get_nodelist(args)
    elif args['allocate']:
        directive = allocate_nodes(args)
    else:
        directive = get_nodes_ppn(args)
    return directive


def get_directives(args: dict) -> None:
    """Collect all the directives for the Torque or Slurm job.
    This results in extending the `args` dictionary with the "directives"
    key, pointing to the list of job directives.

    Parameters
    ----------
    args : dict
        All arguments
    """
    # init directives with a shebang that's hopefully universal here
    args['directives'] = ['#!/bin/bash']
    for set_directive in (
        set_environment,
        set_account,
        set_partition,
        set_job,
        set_localscratch,
        set_email,
        set_stdout_stderr,
        set_time,
        set_memory,
        set_nodes
    ):
        directive = set_directive(args)
        if directive:
            args['directives'].append(directive)



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import unittest
from unittest import mock
from unittest.mock import patch
from datetime import datetime as dt

from pandas.testing import assert_frame_equal
from Xhpc.io_utils import *

import pkg_resources
test_folder = pkg_resources.resource_filename("Xhpc", "test")


class TestSinfo(unittest.TestCase):

    def setUp(self) -> None:
        self.sinfo_dir = '%s/.dummy_sinfo' % test_folder
        os.makedirs(self.sinfo_dir)

        self.config_folder = '%s/configs' % test_folder
        os.makedirs(self.config_folder)
        self.path1 = '%s/file1.txt' % self.config_folder
        self.path2 = '%s/file2.txt' % self.config_folder
        with open(self.path1, 'w') as o1, open(self.path2, 'w') as o2:
            o1.write('line1\nline2\nline3\n')
            o2.write('\n')

        self.today_fp = '%s/%s.tsv' % (self.sinfo_dir, str(dt.now().date()))

        self.sinfo_fps = []
        for day in [1, 2, 3]:
            self.sinfo = '%s/2022-01-0%s.tsv' % (self.sinfo_dir, day)
            with open(self.sinfo, 'w'):
                pass
            self.sinfo_fps.append(self.sinfo)

        self.stdout_bytes = b'some\ttab\tand\nreturn\tseparated\toutput\n'
        self.stdout_lists = [['some', 'tab', 'and'],
                             ['return', 'separated', 'output']]
        self.stdout_10cols = b'1 2 3 4 5 6 7 8 9 10\na b c d e f g h i j\n'
        self.stdout_pd = pd.DataFrame(
            [['1', '2', '3', '4', '5', '6', '7', '8', '9', '10'],
             ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j']],
            columns=['node', 'partition', 'status', 'cpu_load', 'cpus',
                     'socket', 'cores', 'threads', 'mem', 'free_mem'])

    def test_get_first_line(self):
        first_line = get_first_line(self.path1)
        self.assertEqual(first_line, 'line1')
        first_line = get_first_line(self.path2)
        self.assertEqual(first_line, '')

    def test_remove_previous_fps(self):
        for sinfo_fp in self.sinfo_fps:
            self.assertTrue(isfile(sinfo_fp))
        remove_previous_fps(self.sinfo_dir)
        for sinfo_fp in self.sinfo_fps:
            self.assertFalse(isfile(sinfo_fp))

    @unittest.mock.patch('subprocess.Popen')
    def test_run_sinfo(self, mock_popen):
        process_mock = unittest.mock.Mock()
        attrs = {'communicate.return_value': ('output', '')}
        process_mock.configure_mock(**attrs)
        mock_popen.return_value = process_mock
        run_sinfo_subprocess()
        self.assertTrue(mock_popen.called)

    @unittest.mock.patch('subprocess.Popen')
    def test_run_no_sinfo(self, mock_popen):
        process_mock = unittest.mock.Mock()
        attrs = {'communicate.return_value': ('', 'error')}
        process_mock.configure_mock(**attrs)
        mock_popen.return_value = process_mock
        with self.assertRaises(OSError) as cm:
            run_sinfo_subprocess()
        self.assertEqual(
            str(cm.exception), 'Using Slurm? `sinfo` command not found')

    def test_decode_sinfo_stdout(self):
        obs = decode_sinfo_stdout(self.stdout_bytes)
        exp = self.stdout_lists
        self.assertEqual(exp, obs)

        obs = decode_sinfo_stdout(b'a b c\nd e f\n')
        exp = [['a', 'b', 'c'], ['d', 'e', 'f']]
        self.assertEqual(exp, obs)

        obs = decode_sinfo_stdout(b'a    b    c\nd    e    f')
        exp = [['a', 'b', 'c'], ['d', 'e', 'f']]
        self.assertEqual(exp, obs)

    @unittest.mock.patch('subprocess.Popen')
    def test_collect_sinfo_pd(self, mock_popen):
        process_mock = unittest.mock.Mock()
        attrs = {'communicate.return_value': (self.stdout_10cols, '')}
        process_mock.configure_mock(**attrs)
        mock_popen.return_value = process_mock
        obs = collect_sinfo_pd()
        exp = self.stdout_pd
        assert_frame_equal(exp, obs)

    @unittest.mock.patch('subprocess.Popen')
    def test_failed_collect_sinfo_pd(self, mock_popen):
        process_mock = unittest.mock.Mock()
        attrs = {'communicate.return_value': ('', 'error')}
        process_mock.configure_mock(**attrs)
        mock_popen.return_value = process_mock
        with self.assertRaises(OSError) as cm:
            collect_sinfo_pd()
        self.assertEqual(
            str(cm.exception), 'Using Slurm? `sinfo` command not found')

    def test_get_sinfo_pd_expection(self):
        exception = 'No node allocation yet avail for PBS/Torque'
        with self.assertRaises(IOError) as cm:
            args = {'sinfo': False, 'allocate': False, 'torque': True}
            get_sinfo_pd(args, '')
        self.assertEqual(str(cm.exception), exception)
        with self.assertRaises(IOError) as cm:
            args = {'sinfo': True, 'allocate': True, 'torque': True}
            get_sinfo_pd(args, '')
        self.assertEqual(str(cm.exception), exception)
        with self.assertRaises(IOError) as cm:
            args = {'sinfo': True, 'allocate': False, 'torque': True}
            get_sinfo_pd(args, '')
        self.assertEqual(str(cm.exception), exception)
        with self.assertRaises(IOError) as cm:
            args = {'sinfo': False, 'allocate': True, 'torque': True}
            get_sinfo_pd(args, '')
        self.assertEqual(str(cm.exception), exception)

    def test_get_sinfo_pd_empty(self):
        args = {'sinfo': False, 'allocate': False, 'torque': False}
        get_sinfo_pd(args, '')
        assert_frame_equal(pd.DataFrame(), args['sinfo_pd'])

    def test_get_sinfo_pd_read(self):
        self.stdout_pd.to_csv(self.today_fp, index=False, sep='\t')
        args = {'sinfo': False, 'allocate': True, 'torque': False}
        get_sinfo_pd(args, self.sinfo_dir)
        exp = self.stdout_pd
        assert_frame_equal(exp, args['sinfo_pd'])
        os.remove(self.today_fp)

    @unittest.mock.patch('subprocess.Popen')
    def test_get_sinfo_pd_write(self, mock_popen):
        process_mock = unittest.mock.Mock()
        attrs = {'communicate.return_value': (self.stdout_10cols, '')}
        process_mock.configure_mock(**attrs)
        mock_popen.return_value = process_mock
        args = {'sinfo': True, 'allocate': False, 'torque': False}
        get_sinfo_pd(args, self.sinfo_dir)
        assert_frame_equal(self.stdout_pd, args['sinfo_pd'])

    def tearDown(self) -> None:
        for sinfo_fp in self.sinfo_fps:
            if isfile(sinfo_fp):
                os.remove(sinfo_fp)
        if isfile(self.today_fp):
            os.remove(self.today_fp)
        os.rmdir(self.sinfo_dir)
        for path in [self.path1, self.path2]:
            if os.path.isfile(path):
                os.remove(path)
        for folder in [self.config_folder]:
            os.rmdir(folder)


class TestOutput(unittest.TestCase):

    def setUp(self) -> None:
        self.pwd = abspath('.')
        self.real_dir = '%s/existing_dir' % test_folder
        os.makedirs(self.real_dir)
        self.fake_dir = '%s/non_existing_dir' % test_folder
        self.time = dt.now().strftime("%Y-%m-%d_%H-%M-%S")

    def test_get_job_fp(self):
        # no output path given and default, slurm usage

        args = {'job': 'a_name', 'output_fp': None, 'torque': False}
        get_job_fp(args)
        exp = '%s/a_name_%s.slm' % (self.pwd, self.time)
        self.assertEqual(exp, args['job_fp'])

        # no output path given and torque usage
        args = {'job': 'a_name', 'output_fp': None, 'torque': True}
        get_job_fp(args)
        exp = '%s/a_name_%s.pbs' % (self.pwd, self.time)
        self.assertEqual(exp, args['job_fp'])

        # output path given and default, slurm usage
        slm_fp = '/a/path/to/should/exists.slm'
        args = {'job': 'a_name', 'output_fp': slm_fp, 'torque': False}
        get_job_fp(args)
        self.assertEqual(slm_fp, args['job_fp'])

        # output path given and torque usage
        pbs_fp = '/a/path/to/should/exists.pbs'
        args = {'job': 'a_name', 'output_fp': pbs_fp, 'torque': True}
        get_job_fp(args)
        self.assertEqual(pbs_fp, args['job_fp'])

    def test_get_output_dir(self):
        args = {'out_dir': self.real_dir}
        get_output_dir(args)
        self.assertEqual(self.real_dir, args['output_dir'])

        args = {'out_dir': self.fake_dir}
        get_output_dir(args)
        self.assertEqual(self.pwd, args['output_dir'])

        args = {'out_dir': 'some/relative/folder'}
        get_output_dir(args)
        self.assertEqual(self.pwd, args['output_dir'])

        args = {'out_dir': 'some_relative_folder'}
        exp = '%s/some_relative_folder' % self.pwd
        os.makedirs(exp)
        get_output_dir(args)
        self.assertEqual(exp, args['output_dir'])
        os.rmdir(exp)

    def tearDown(self) -> None:
        os.rmdir(self.real_dir)


class TestCheckContent(unittest.TestCase):

    def setUp(self) -> None:
        self.args = {'directives': ['A'], 'preamble': ['B'],
                     'commands': ['C'], 'verif': True}

    def test_get_lines(self):
        obs = get_lines(['2', '1'])
        exp = ['2', '1']
        self.assertEqual(exp, obs)

        obs = get_lines({'2', '1'})
        exp = ['1', '2']
        self.assertEqual(exp, obs)

        with self.assertRaises(TypeError) as cm:
            get_lines('a')
        self.assertEqual(str(cm.exception),
                         'Commands are collected either as list or set')

    def test_no_check_content(self):
        self.args = {'verif': False}
        self.assertIsNone(check_content(self.args))

    @patch('builtins.input', return_value='n')
    def test_check_content(self, input):
        with self.assertRaises(SystemExit) as cm:
            check_content(self.args)
        self.assertEqual(cm.exception.code, 1)


class TestWriteOut(unittest.TestCase):

    def setUp(self) -> None:
        self.out = '%s/dummy_output.txt' % test_folder
        self.content_base = [
            'A\n', '# ------ directives END ------\n', '\n',
            'B\n', '# ------ preamble END ------\n', '\n',
            'D\n', '# ------ commands END ------\n', '\n',
            'echo "Done!"\n']
        self.content_time = [
            'A\n', '# ------ directives END ------\n', '\n',
            'B\n', '# ------ preamble END ------\n', '\n',
            '/usr/bin/time -v cmd \\\n', '\t-i X \\\n', '\t-e Y\n',
            '# ------ commands END ------\n', '\n',
            'echo "Done!"\n']
        self.content_move = [
            'A\n', '# ------ directives END ------\n', '\n',
            'B\n', '# ------ preamble END ------\n', '\n',
            'C\n', '# ------ move_to END ------\n', '\n',
            'D\n', '# ------ commands END ------\n', '\n',
            'E\n', '# ------ move_from END ------\n', '\n',
            'F\n', '# ------ clear END ------\n', '\n',
            'echo "Done!"\n']
        self.args = {
            'job_fp': self.out,
            'directives': {'A'},
            'preamble': {'B'},
            'move_to': set(),
            'commands': ['D'],
            'move_from': set(),
            'clear': set(),
            'move': False,
            'stat': False,
        }

    def test_write_out(self):
        write_out(self.args)
        lines = []
        with open(self.out) as f:
            for line in f:
                lines.append(line)
        self.assertEqual(lines, self.content_base)

    def test_write_out_time(self):
        self.args['stat'] = True
        self.args['commands'] = ['cmd \\', '\t-i X \\', '\t-e Y']
        write_out(self.args)
        lines = []
        with open(self.out) as f:
            for line in f:
                lines.append(line)
        self.assertEqual(lines, self.content_time)

    def test_write_out_move(self):
        self.args['move'] = True
        self.args['move_to'] = {'C'}
        self.args['move_from'] = {'E'}
        self.args['clear'] = {'F'}
        write_out(self.args)
        lines = []
        with open(self.out) as f:
            for line in f:
                lines.append(line)
        self.assertEqual(lines, self.content_move)

    def tearDown(self) -> None:
        os.remove(self.out)


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import re
import sys
from os.path import abspath, exists, isfile


def get_abspath(term_: str) -> str:
    """Get the absolute path of an existing path but keep the trailing
    slash if it was present in the relative / user-defined path (can be
    needed for command such as rsync).

    Parameters
    ----------
    term_ : str
        A path

    Returns
    -------
    term : str
        The abspath of the path
    """
    term = abspath(term_)
    if term_.endswith('/'):
        term += '/'
    return term


def check_term(args: dict, term_: str) -> str:
    """If the term is a character string starting with an alphanumeric
    character, then check here whether it is an existing file/folder not yet
    in absolute path format, or an executable. If it is an executable (i.e.,
    could well be a local executable not present in a PATH), it is preserved
    unchanged in the parsed command line, otherwise, it is replaced to it
    abspath.
    This is somewhat redundant with `get_term` but would speed up by avoiding
    looking up in the set of executables the term using complicated  relative
    paths (i.e.,  that do not start with an alphanumeric character).

    Parameters
    ----------
    args : dict
        All arguments
    term_ : str
        A space-separated term in a decomposed command line

    Returns
    -------
    term : str
        The space-separated term changed (or not) to abspath
    """
    if term_ in args['executables']:
        term = term_
    elif exists(term_):
        term = get_abspath(term_)
    else:
        term = term_
    return term


def get_term(args: dict, tdx: int, term_: str) -> str:
    """Get the abspath of the input command line term if it is a path
    and not an executable. The case of an empty term may arise when the
    command line contained quotes that were split.

    Parameters
    ----------
    args : dict
        All arguments
    tdx : int
        Index of the term in the command line
    term_ : str
        A space-separated term in a decomposed command line

    Returns
    -------
    term : str
        The space-separated term changed (or not) to abspath
    """
    term = ''
    if term_:
        if tdx == 0 or re.match('[a-zA-Z0-9]', term_):
            term = check_term(args, term_)
        elif exists(term_):
            term = get_abspath(term_)
        else:
            term = term_
    return term


def split_quoted(line: str, quote: str) -> list:
    """Split a line based on the quotes and labels the ports that were
    between quotes using a two-items tuple, with the first item being the
    split part and the second item being the quote.

    Parameters
    ----------
    line : str
        Command line that contains some quotes
    quote : str
        A simple or double quote symbol

    Returns
    -------
    line_split : list

    """
    ls = line.split(quote)
    line_split = [(y, quote) if x % 2 else (y, '') for x, y in enumerate(ls)]
    return line_split


def quote_split_line(line_: str) -> list:
    """Split a command line that may or may not contain quotes. If quotes
    are present, it is important to split on quotes too in order to identify
    potential quoted paths that would need to be changes to abspath.

    Parameters
    ----------
    line_ : str
        Command line to split

    Returns
    -------
    split_line : list
        Split command line where items are tuples such as (term, quoting)
    """
    if '"' in line_:
        line_split = split_quoted(line_, '"')
    elif "'" in line_:
        line_split = split_quoted(line_, "'")
    else:
        line_split = [(line_, '')]
    return line_split


def get_final_command_part(quote_part: list, quote: str) -> str:
    """Reconstitute the part of the command line that was split
    based on quoting (or not). If a split was indeed operated based
    on quoting, add the quote used to split back, around the
    expression.

    Parameters
    ----------
    quote_part : list
        Element of a command line that were split based on quoting
    quote : str
        Either " or ' quote character

    Returns
    -------
    final_part : str
        Reconstituted command line
    """
    final_part = ' '.join(quote_part)
    if quote:
        final_part = quote + final_part + quote
    return final_part


def get_separated_terms(part: str, separator: str) -> list:
    """Get the list of terms that may hide separated by a comma in the command.

    Parameters
    ----------
    part : str
        A space-separated term in a decomposed command line
    separator : str
        A comma or a colon or a semicolon

    Returns
    -------
    terms : list
        Either the term alone or the terms separated by a comma
    """
    terms = [part]
    if separator in part:
        terms = part.split(separator)
    return terms


def parse_line(line_: str, args: dict, paths: set, commands: list) -> None:
    """Turn the words that have a path into an abspath (except for
    executables), and prepend these with the environment variable pointing
    to the scratch folder is the option to move files to scratch is set.

    Parameters
    ----------
    line_ : str
        Current line of the script file.
    args : dict
        All arguments. Here only the following keys are of interest:
            input_fp : str
                Input script path (or double-quoted command)
            executables : set
                Conda and global paths executables
            abs : bool
                Change the existing paths in command line to absolute paths
    paths : set
        Paths to file or folders to move when using scratch folder
    commands : list
        Line of the script with paths potentially changed to abspaths
    """
    if not line_.strip() or line_[0] == '#':
        commands.append(line_.strip('\n'))
    else:
        parts = list()
        for (quote_part_, quote) in quote_split_line(line_.strip('\n')):
            quote_part = list()
            quote_part_split = quote_part_.split(' ')
            for tdx, part in enumerate(quote_part_split):
                comma_terms = get_separated_terms(part, ',')
                comma_separated = []
                for comma_term in comma_terms:
                    colon_terms = get_separated_terms(comma_term, ':')
                    colon_separated = []
                    for term_ in colon_terms:
                        # Get the term (an abspath if it is an existing path)
                        if term_.startswith('${SCRATCH_FOLDER}'):
                            term = term_
                        elif args['abspath']:
                            term = get_term(args, tdx, term_)
                        else:
                            term = term_
                        # If them found as being an absolute path
                        if term.startswith('/'):
                            # Add term to set of paths
                            paths.add(term)
                            # Use it in command from scratch area if requested
                            if args['move'] and term not in args['exclude']:
                                term = '${SCRATCH_FOLDER}%s' % term
                        colon_separated.append(term)
                    comma_separated.append(':'.join(colon_separated))
                quote_part.append(','.join(comma_separated))
            parts.append(get_final_command_part(quote_part, quote))
        commands.append(''.join(parts))


def parse_script(args: dict) -> None:
    """Parse the .sh file and collect the actual script commands.
    This results in extending the `args` dictionary with the "commands" and
    "paths" keys, pointing to the list of commands and sets of files/folders
    to move to/from scratch locations.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            input_fp : str
                Input script path (or double-quoted command)
            abspath : bool
                Parse command and change existing paths to abspath
    """
    paths = set()
    commands = list()
    with open(args['input_fp']) as f:
        # for each command of the script
        for line_ in f:
            if args['abspath'] or args['move']:
                # collect abspath for existing files/folders or keep words as is
                parse_line(line_.strip(), args, paths, commands)
            else:
                commands.append(line_)
    args['commands'] = commands
    args['paths'] = paths


def get_commands(args: dict) -> None:
    """Check whether the command is passed as a script file or directly as
    an inline command, and parse the commands to collect in the `args` object
    the commands and the paths of files/folders to potentially move to scratch.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            input_fp : str
                Input script path (or double-quoted command)
    """
    if isfile(args['input_fp']):
        # if the script file exists
        parse_script(args)  # get commands and ins/outs from file content
    else:
        # if the script file does not exist: could be a direct command line
        print('Command line parsing not yet available')
        sys.exit(1)



# ----------------------------------------------------------------------------
# Copyright (c) 2020, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import unittest
import pandas as pd
from Xhpc.directives import *
import pkg_resources
test_folder = pkg_resources.resource_filename("Xhpc", "test")


class TestConda(unittest.TestCase):

    def setUp(self):
        self.args = {
            'job': 'job_name',
            'account': 'account_name',
            'partition': 'normal',
            'workdir': '/some/path/here',
            'time': '10',
            'gpu': False,
            'torque': False,
            'email': True,
            'nodes': None,
            'allocate': None,
            'nnodes': 1,
            'cpus': 1,
            'sinfo_pd': pd.DataFrame()
        }
        self.email_address = 'some@email.com'

    def test_set_account(self):
        exp = '#SBATCH --account=%s' % self.args['account']
        obs = set_account(self.args)
        self.assertEqual(exp, obs)

        self.args['torque'] = True
        exp = '#PBS -A %s' % self.args['account']
        obs = set_account(self.args)
        self.assertEqual(exp, obs)

        self.args['account'] = None
        exp = None
        obs = set_account(self.args)
        self.assertEqual(exp, obs)

    def test_set_partition(self):
        exp = '#SBATCH --partition=normal'
        obs = set_partition(self.args)
        self.assertEqual(exp, obs)

        self.args['partition'] = 'whatever'
        exp = '#SBATCH --partition=whatever'
        obs = set_partition(self.args)
        self.assertEqual(exp, obs)

        self.args['gpu'] = True
        exp = '#SBATCH --partition=accel'
        obs = set_partition(self.args)
        self.assertEqual(exp, obs)

        self.args['torque'] = True
        exp = '#PBS -q whatever'
        obs = set_partition(self.args)
        self.assertEqual(exp, obs)

    def test_set_environment(self):
        exp = '#SBATCH --export=ALL'
        obs = set_environment(self.args)
        self.assertEqual(exp, obs)

        self.args['torque'] = True
        exp = '#PBS -V'
        obs = set_environment(self.args)
        self.assertEqual(exp, obs)

    def test_set_nodes_slurm(self):
        exp = '#SBATCH --ntasks=1'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['nnodes'] = 2
        exp = '#SBATCH --nodes=2\n#SBATCH --ntasks-per-node=1'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['cpus'] = 2
        exp = '#SBATCH --nodes=2\n#SBATCH --ntasks-per-node=2'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['nnodes'] = 1
        exp = '#SBATCH --ntasks=2'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

    def test_set_nodes_allocate(self):
        self.args['allocate'] = True

    def test_set_nodes_names(self):
        self.args['nodes'] = ('a', 'b', 'c',)
        exp = '#SBATCH --nodelist=a,b,c\n#SBATCH --ntasks-per-node=1'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['torque'] = True
        exp = '#PBS -l nodes=a,b,c\n#PBS -l ppn=1'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

    def test_set_nodes_torque(self):
        self.args['torque'] = True
        exp = '#PBS -l nodes=1:ppn=1'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['nnodes'] = 2
        exp = '#PBS -l nodes=2:ppn=1'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['cpus'] = 2
        exp = '#PBS -l nodes=2:ppn=2'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

        self.args['nnodes'] = 1
        exp = '#PBS -l nodes=1:ppn=2'
        obs = set_nodes(self.args)
        self.assertEqual(exp, obs)

    # def test_set_email(self):
    #     obs = set_email(self.args)
    #     self.assertEqual(exp, obs)
    #
    # def test_set_time(self):
    #     obs = set_time(self.args)
    #     self.assertEqual(exp, obs)
    #
    # def test_set_job(self):
    #     obs = set_job(self.args)
    #     self.assertEqual(exp, obs)
    #
    # def test_set_memory(self):
    #     obs = set_memory(self.args)
    #     self.assertEqual(exp, obs)
    #
    # def test_set_stdout_stderr(self):
    #     obs = set_stdout_stderr(self.args)
    #     self.assertEqual(exp, obs)
    #
    # def test_get_directives(self):
    #     obs = get_directives(self.args)
    #     self.assertEqual(exp, obs)


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------
import os

from os.path import dirname, isdir, isfile
from Xhpc.io_utils import get_first_line


def get_email_address(args: dict, config_fp: str) -> None:
    """
    Collect the email address from the edited config file.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following key is of interest:
        config_email: bool
            Show the current email and potentially edit it
        Path to the file that may (or not) contains an email address
    config_fp : str
        Path to the file that may (or not) contains an email address
    """
    if isfile(config_fp):
        email_address = edit_config(config_fp, args['config_email'])
    else:
        email_address = create_config(config_fp)
    args['email_address'] = email_address


def validate_email(email: str) -> bool:
    """A not-so-fancy that a string may be a valid email address.

    Parameters
    ----------
    email : str
        A string that maz look like an email address

    Returns
    -------
    bool
        True if the string looks like an email, else False
    """
    if email.count('@') != 1:
        return False
    user, domain = email.split('@')
    if not len(user):
        return False
    if '.' not in domain:
        return False
    domain_split = domain.split('.')
    if 0 in map(len, domain_split):
        return False
    return True


def get_email() -> str:
    """Collect the email address interactively from the user.

    Returns
    -------
    email : str
        email address of the user.
    """
    email = input('Please enter a valid email address: ')
    if not validate_email(email):
        raise ValueError('%s not a valid email address' % email)
    return email


def edit_config(config_fp: str, config_email: bool):
    """
    Checks that the content of the existing config file
    is a valid email address.

    Parameters
    ----------
    config_fp : str
        Path to the file that may (or not) contains an email address
    config_email: bool
        Show the current email and potentially edit it

    Returns
    -------
    email : str
        email address of the user.
    """
    # parse the first line of the config file
    email = get_first_line(config_fp)
    if config_email:
        print("Registered email:", email)
        email = create_config(config_fp)
    return email


def create_config(config_fp: str) -> str:
    """Collect the email address interactively from the user
    and write it somewhere it can be reused.

    Parameters
    ----------
    config_fp : str
        Path to the config file

    Returns
    -------
    email : str
        email address of the user.
    """
    email = get_email()
    write_email(config_fp, email)
    return email


def write_email(config_fp: str, email: str):
    """Write the email address somewhere it can be reused.

    Parameters
    ----------
    config_fp : str
        Path to the config file
    email : str
        email address of the user.
    """
    if not isdir(dirname(config_fp)):
        os.makedirs(dirname(config_fp))
    with open(config_fp, 'w') as o:
        o.write('%s\n' % email)
        o.close()
    print('Written: %s' % config_fp)



# ----------------------------------------------------------------------------
# Copyright (c) 2020, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------
import os
import unittest
from Xhpc.cmd import *
import pkg_resources
test_folder = pkg_resources.resource_filename("Xhpc", "test")


class Test(unittest.TestCase):

    def setUp(self):
        self.args = {
            'executables': {'exe'},
            'move': False
        }
        self.path = './test_check_term.txt'
        with open(self.path, 'w') as o:
            pass
        self.abspath = os.path.abspath(self.path)
        self.paths = set()
        self.line_comment = '# line'
        self.line_simple = 'exe -i in -o out'
        self.line_simple_path = 'exe -i /in -o /out'
        self.line_exists = 'exe -i test_parse_line_in -o test_parse_line_out'
        with open('test_parse_line_in', 'w'):
            pass
        self.abspath_fp = os.path.abspath('test_parse_line_in')
        self.commands = []

        self.line_comment_fp = '%s/line_comment_fp.txt' % test_folder
        with open(self.line_comment_fp, 'w') as o:
            o.write('%s\n' % self.line_comment)
        self.line_simple_fp = '%s/line_simple_fp.txt' % test_folder
        with open(self.line_simple_fp, 'w') as o:
            o.write('%s\n' % self.line_simple)
        self.line_simple_path_fp = '%s/line_simple_path_fp.txt' % test_folder
        with open(self.line_simple_path_fp, 'w') as o:
            o.write('%s\n' % self.line_simple_path)
        self.line_exists_fp = '%s/line_exists_fp.txt' % test_folder
        with open(self.line_exists_fp, 'w') as o:
            o.write('%s\n' % self.line_exists)

    def test_check_term(self):
        obs = check_term(self.args, 'exe')
        self.assertEqual(obs, 'exe')
        obs = check_term(self.args, self.path)
        self.assertEqual(obs, self.abspath)
        obs = check_term(self.args, 'whatever')
        self.assertEqual(obs, 'whatever')

    def test_get_separated_terms(self):
        term, sep = 'no_sep', ','
        exp = ['no_sep']
        obs = get_separated_terms(term, sep)
        self.assertEqual(exp, obs)

        term, sep = 'sep1,sep2', ','
        exp = ['sep1', 'sep2']
        obs = get_separated_terms(term, sep)
        self.assertEqual(exp, obs)

        term, sep = 'sep1:sep2', ':'
        exp = ['sep1', 'sep2']
        obs = get_separated_terms(term, sep)
        self.assertEqual(exp, obs)

    def test_get_term(self):
        obs = get_term(self.args, 0, '')
        self.assertEqual(obs, '')
        obs = get_term(self.args, 0, 'exe')
        self.assertEqual(obs, 'exe')
        obs = get_term(self.args, 0, self.path)
        self.assertEqual(obs, self.abspath)
        obs = get_term(self.args, 0, 'whatever')
        self.assertEqual(obs, 'whatever')
        obs = get_term(self.args, 1, '')
        self.assertEqual(obs, '')
        obs = get_term(self.args, 1, 'exe')
        self.assertEqual(obs, 'exe')
        obs = get_term(self.args, 1, self.path)
        self.assertEqual(obs, self.abspath)
        obs = get_term(self.args, 1, '/whatever')
        self.assertEqual(obs, '/whatever')

    def test_split_quoted(self):
        obs = split_quoted('say "hello" here', '"')
        exp = [('say ', ''), ('hello', '"'), (' here', '')]
        self.assertEqual(exp, obs)
        obs = split_quoted("say 'hello' here", "'")
        exp = [('say ', ''), ('hello', "'"), (' here', '')]
        self.assertEqual(exp, obs)
        obs = split_quoted("say 'hello' here 'again'", "'")
        exp = [('say ', ''), ('hello', "'"), (' here ', ''),
               ('again', "'"), ('', '')]
        self.assertEqual(exp, obs)
        obs = split_quoted("'that' should never happen", "'")
        exp = [('', ''), ('that', "'"), (' should never happen', '')]
        self.assertEqual(exp, obs)

    def test_quote_split_line(self):
        obs = quote_split_line('say hello here')
        exp = [('say hello here', '')]
        self.assertEqual(exp, obs)
        obs = quote_split_line('say "hello" here')
        exp = [('say ', ''), ('hello', '"'), (' here', '')]
        self.assertEqual(exp, obs)
        obs = quote_split_line("say 'hello' here")
        exp = [('say ', ''), ('hello', "'"), (' here', '')]
        self.assertEqual(exp, obs)
        obs = quote_split_line("say 'hello' here 'again'")
        exp = [('say ', ''), ('hello', "'"), (' here ', ''),
               ('again', "'"), ('', '')]
        self.assertEqual(exp, obs)
        obs = quote_split_line("'that' should never happen")
        exp = [('', ''), ('that', "'"), (' should never happen', '')]
        self.assertEqual(exp, obs)

    def test_get_final_command_part(self):
        obs = get_final_command_part(['a', 'b'], '')
        exp = 'a b'
        self.assertEqual(exp, obs)
        obs = get_final_command_part(['a', 'b'], '"')
        exp = '"a b"'
        self.assertEqual(exp, obs)
        obs = get_final_command_part(['a', 'b'], "'")
        exp = "'a b'"
        self.assertEqual(exp, obs)

    def test_parse_line_comment(self):
        parse_line(self.line_comment, self.args, self.paths, self.commands)
        self.assertEqual(['# line'], self.commands)

    def test_parse_line_simple(self):
        parse_line(self.line_simple, self.args, self.paths, self.commands)
        self.assertEqual([self.line_simple], self.commands)
        self.assertEqual(self.paths, set())

    def test_parse_line_simple_path(self):
        line_simple_path = 'exe -i /in -o /out'
        parse_line(line_simple_path, self.args, self.paths, self.commands)
        self.assertEqual([line_simple_path], self.commands)
        self.assertEqual(self.paths, {'/in', '/out'})

    def test_parse_line_simple_path_move(self):
        self.args['move'] = True
        line_simple_path = 'exe -i /in -o /out'
        parse_line(line_simple_path, self.args, self.paths, self.commands)
        self.assertEqual(['exe -i ${SCRATCH_DIR}/in -o ${SCRATCH_DIR}/out'],
                         self.commands)
        self.assertEqual(self.paths, {'/in', '/out'})

    def test_parse_line_exists(self):
        line_exists = 'exe -i test_parse_line_in -o test_parse_line_out'
        parse_line(line_exists, self.args, self.paths, self.commands)
        exp = ['exe -i %s -o test_parse_line_out' % self.abspath_fp]
        self.assertEqual(exp, self.commands)
        self.assertEqual(self.paths, {self.abspath_fp})

    def test_parse_line_complex(self):
        self.args['move'] = True
        line = 'name -i "/a,/b" -o /x,/y:/1:/2'
        parse_line(line, self.args, self.paths, self.commands)
        out = 'name -i "${SCRATCH_DIR}/a,${SCRATCH_DIR}/b" -o ${SCRATCH_DIR}'
        out += '/x,${SCRATCH_DIR}/y:${SCRATCH_DIR}/1:${SCRATCH_DIR}/2'
        exp = [out]
        self.assertEqual(exp, self.commands)
        self.assertEqual(self.paths, {'/a', '/b', '/x', '/y', '/1', '/2'})

    def test_parse_line_exists_move(self):
        self.args['move'] = True
        line_exists = 'exe -i test_parse_line_in -o test_parse_line_out'
        parse_line(line_exists, self.args, self.paths, self.commands)
        exp = [
            'exe -i ${SCRATCH_DIR}%s -o test_parse_line_out' % self.abspath_fp]
        self.assertEqual(exp, self.commands)
        self.assertEqual(self.paths, {self.abspath_fp})

    def test_get_commands_comment(self):
        self.args['input_fp'] = self.line_comment_fp
        get_commands(self.args)
        self.assertEqual(['# line'], self.args['commands'])

    def test_get_commands_simple(self):
        self.args['input_fp'] = self.line_simple_fp
        get_commands(self.args)
        self.assertEqual([self.line_simple], self.args['commands'])
        self.assertEqual(self.args['paths'], set())

    def test_get_commands_simple_path(self):
        self.args['input_fp'] = self.line_simple_path_fp
        get_commands(self.args)
        self.assertEqual([self.line_simple_path], self.args['commands'])
        self.assertEqual(self.args['paths'], {'/in', '/out'})

    def test_get_commands_exists(self):
        self.args['input_fp'] = self.line_exists_fp
        get_commands(self.args)
        exp = ['exe -i %s -o test_parse_line_out' % self.abspath_fp]
        self.assertEqual(exp, self.args['commands'])
        self.assertEqual(self.args['paths'], {self.abspath_fp})

    def test_get_commands_exists_move(self):
        self.args['input_fp'] = self.line_exists_fp
        self.args['move'] = True
        get_commands(self.args)
        exp = [
            'exe -i ${SCRATCH_DIR}%s -o test_parse_line_out' % self.abspath_fp]
        self.assertEqual(exp, self.args['commands'])
        self.assertEqual(self.args['paths'], {self.abspath_fp})

    def tearDown(self) -> None:
        os.remove(self.path)
        if isfile(self.abspath):
            os.remove(self.abspath)
        os.remove(self.line_comment_fp)
        os.remove(self.line_simple_fp)
        os.remove(self.line_simple_path_fp)
        os.remove(self.line_exists_fp)


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the MIT License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import click

from Xhpc.xhpc import xhpc
from Xhpc import __version__


@click.command()
@click.option(
    "-i", "--i-script", default=None, nargs=1,
    help="Input script path (or double-quoted command)")
@click.option(
    "-o", "--o-script", default=None, type=str,
    help="Output script path (default to <input>_<YYYY-MM-DD-HH-MM-SS>.slm)")
@click.option(
    "-j", "--i-job", default=None, type=str, help="Job name")
@click.option(
    "-a", "--p-account", default=None, type=str, help="Name of the account")
@click.option(
    "-p", "--p-partition", default='normal', help="Partition name",
    type=click.Choice(['normal', 'bigmem', 'accel', 'optimist']),)
@click.option(
    "-n", "--p-nnodes", default=1, type=int,
    help="Number of nodes", show_default=True)
@click.option(
    "-c", "--p-cpus", default=1, type=int,
    help="Number of CPUs", show_default=True)
@click.option(
    "-t", "--p-time", default=1, type=int,
    help="Wall time limit (in hours)", show_default=True)
@click.option(
    "-M", "--p-mem", nargs=2, show_default=False, default=('500', 'MB'),
    help="Requested memory as two space-separated entries: an integer and "
         "either 'MB' or 'GB'. (Default: '500 MB')")
@click.option(
    "-m", "--p-mem-per-cpu", nargs=2, show_default=False, default=None,
    help="Requested memory per cpu as two space-separated entries: an integer "
         "and either 'MB' or 'GB'. (Default: '500 MB')")
@click.option(
    "-N", "--p-nodes", default=None, multiple=True,
    help="Node names, e.g. `-N c1-4 -N c6-10 -N c7-1` (overrides option `-n`)")
@click.option(
    "-e", "--p-env", default=None, type=str,
    help="Conda environment to run the job")
@click.option(
    "-d", "--p-dir", help="Output directory", type=str,
    default='.', show_default=True)
@click.option(
    "-T", "--p-tmp", default=None, type=str,
    help="Alternative temp folder to the one defined in $TMPDIR (if not "
         "defined: will be set to $USERWORK, or to $SCRATCH if any scratch "
         "option is activated)")
@click.option(
    "-w", "--p-workdir", default=None, show_default=True,
    help="Working directory to use instead of $SLURM_SUBMIT_DIR")
@click.option(
    "-y", "--p-include", multiple=True, default=None, show_default=True,
    help="Folder to not move to and from scratch using rsync (must exist)")
@click.option(
    "-x", "--p-exclude", multiple=True, default=None, show_default=True,
    help="Relative path(s) within input folder(s) to not move in scratch")
@click.option(
    "--move/--no-move", default=False, show_default=True,
    help="Move files/folders to chosen scratch location")
@click.option(
    "--abspath/--no-abspath", default=True, show_default=True,
    help="Change the existing paths in command line to absolute paths")
@click.option(
    "-l", "--localscratch", type=int, show_default=False, default=None,
    help="Use localscratch with the provided memory amount (in GB)")
@click.option(
    "--scratch/--no-scratch", default=False, show_default=True,
    help="Use the scratch folder to move files and compute")
@click.option(
    "--userscratch/--no-userscratch", default=False, show_default=True,
    help="Use the userscratch folder to move files and compute")
@click.option(
    "--clear-scratch/--no-clear-scratch", default=True, show_default=True,
    help="Whether to cleat the scratch area at the end of the job or not")
@click.option(
    "--stdout/--no-stdout", default=True, show_default=True,
    help="Rename stdout (and stderr) with job name and ID")
@click.option(
    "--email/--no-email", default=False, show_default=True,
    help="Send email at job completion (always if fail)")
@click.option(
    "--run/--no-run", default=False, show_default=True,
    help="Run the job before exiting (subprocess)")
@click.option(
    "--stat/--no-stat", default=True, show_default=True,
    help="Whether to prepend `/usr/bin/time -v` to every script command")
@click.option(
    "--verif/--no-verif", default=False, show_default=True,
    help="Print script to stdout and ask for 'y/n' user input to sanity check")
@click.option(
    "--gpu/--no-gpu", default=False, show_default=True,
    help="Query a gpu (experimental)")
@click.option(
    "--torque/--no-torque", default=False, show_default=True,
    help="Adapt to Torque")
@click.option(
    "--config-email/--no-config-email", default=False, show_default=True,
    help="Show current email and/or edit it")
@click.option(
    "--config-scratch/--no-config-scratch", default=False,
    show_default=True, help="Show current scratch folders and/or edit them")
@click.option(
    "--sinfo/--no-sinfo", default=False, show_default=True,
    help="Print sinfo in stdout (and update `~/.sinfo`)")
@click.option(
    "--allocate/--no-allocate", default=False, show_default=True,
    help="Get current machine usage (sinfo) to allocate suitable nodes/memory")
@click.option(
    "--show-config/--no-show-config", default=False, show_default=True,
    help="Show current configurations (email and scratches)")
@click.option(
    "--saga/--no-saga", default=False, show_default=True,
    help="Import conda env as specified on sigma2/SAGA documentation.")
@click.option(
    "--quiet/--no-quiet", default=False, show_default=True,
    help="Do not print anything")
@click.version_option(__version__, prog_name="Xhpc")


def standalone_xhpc(
        i_script,
        i_job,
        o_script,
        p_account,
        p_partition,
        p_env,
        p_dir,
        p_nnodes,
        p_cpus,
        p_time,
        p_tmp,
        p_mem,
        p_mem_per_cpu,
        p_nodes,
        p_workdir,
        p_include,
        p_exclude,
        localscratch,
        scratch,
        userscratch,
        clear_scratch,
        stdout,
        email,
        run,
        move,
        abspath,
        verif,
        stat,
        gpu,
        torque,
        config_email,
        config_scratch,
        sinfo,
        allocate,
        show_config,
        saga,
        quiet
):

    xhpc(
        input_fp=i_script,
        job=i_job,
        output_fp=o_script,
        account=p_account,
        partition=p_partition,
        env=p_env,
        out_dir=p_dir,
        nnodes=p_nnodes,
        cpus=p_cpus,
        time=p_time,
        tmp=p_tmp,
        mem=p_mem,
        mem_per_cpu=p_mem_per_cpu,
        nodes=p_nodes,
        include=p_include,
        exclude=p_exclude,
        scratch=scratch,
        localscratch=localscratch,
        userscratch=userscratch,
        clear_scratch=clear_scratch,
        workdir=p_workdir,
        stdout=stdout,
        email=email,
        run=run,
        move=move,
        abspath=abspath,
        verif=verif,
        stat=stat,
        gpu=gpu,
        torque=torque,
        config_email=config_email,
        config_scratch=config_scratch,
        sinfo=sinfo,
        allocate=allocate,
        show_config=show_config,
        saga=saga,
        quiet=quiet
    )


if __name__ == "__main__":
    standalone_xhpc()



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import pandas as pd


def get_nodelist(args: dict) -> str:
    """Get the directive for the listed nodes.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            nodes: tuple
                Node names
            torque: bool
                Adapt to Torque
            sinfo_pd : pd.DataFrame
                Per-node sinfo output for an extensive set of parameters

    Returns
    -------
    directive : str
        Current directive
    """
    if args['torque']:
        directive = '#PBS -l nodes=%s' % ','.join(args['nodes'])
        directive += '\n#PBS -l ppn=%s' % args['cpus']
    else:
        directive = '#SBATCH --nodelist=%s' % ','.join(args['nodes'])
        directive += '\n#SBATCH --ntasks-per-node=%s' % args['cpus']
    return directive


def get_nodes_ppn(args: dict) -> str:
    """Distribute the number of processors requested among the requested nodes.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            nnodes: int
                Number of nodes
            cpus: int
                Number of CPUs
            torque: bool
                Adapt to Torque
            sinfo_pd : pd.DataFrame
                Per-node sinfo output for an extensive set of parameters

    Returns
    -------
    directive : str
        Current directive
    """
    nnodes = int(args['nnodes'])  # number of nodes requested
    ncpus = int(args['cpus'])     # number of processors requested
    if args['torque']:
        directive = '#PBS -l nodes=%s:ppn=%s' % (nnodes, ncpus)
    else:
        if nnodes > 1 and ncpus > 1:
            directive = '#SBATCH --nodes=%s' % nnodes
            directive += '\n#SBATCH --ntasks-per-node=%s' % ncpus
        elif nnodes > 1:
            directive = '#SBATCH --nodes=%s' % nnodes
            directive += '\n#SBATCH --ntasks-per-node=1'
        elif ncpus > 1:
            directive = '#SBATCH --ntasks=%s' % ncpus
        else:
            directive = '#SBATCH --ntasks=1'
    return directive


def allocate_nodes(args: dict) -> str:
    """Look at the latest nodes/processors availability and make an
    allocation of such resources to satisfy the requested resources.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            nnodes: int
                Number of nodes
            sinfo_pd : pd.DataFrame
                Per-node sinfo output for an extensive set of parameters

    Returns
    -------
    directive : str
    """
    if args['sinfo_pd'].shape[0]:
        part_pd = subset_sinfo_pd(args)
        directive = get_allocated_nodes(args, part_pd)
    else:
        raise ValueError('There must have been an issue with collecting sinfo')
    return directive


def subset_sinfo_pd(args: dict) -> pd.DataFrame:
    """Subset the sinfo table to the requested partition and .

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            nnodes: int
                Number of nodes
            cpus: int
                Number of CPUs
            sinfo_pd : pd.DataFrame
                Per-node sinfo output for an extensive set of parameters

    Returns
    -------
    part_pd : pd.DataFrame
        sinfo_pd reduced to the queried partition and with expanded procs info
    """
    sinfo_pd = args['sinfo_pd']
    partition = args['partition']
    part_pd = sinfo_pd.loc[sinfo_pd.status.isin(['idle', 'mixed'])].copy()
    if not part_pd.shape[0]:
        raise ValueError(
            'No processors for allocation. Use `--sinfo` to update node info, '
            'or do not use `--allocate` to let Slurm scheduling your job...')
    part_pd = part_pd.loc[part_pd.partition.str.contains(partition)].copy()
    part_pd['mem_load'] = round(100*(1-(part_pd['free_mem']/part_pd['mem'])), 2)
    part_pd['idle'] = part_pd.cpus.apply(lambda x: x.split('/')[1])
    part_pd = part_pd.sort_values('idle', ascending=False)
    return part_pd


def get_allocated_nodes(args: dict, part_pd: pd.DataFrame) -> str:
    """Look at the latest nodes/processors availability and make an
    allocation of such resources to satisfy the requested resources.

    Parameters
    ----------
    args : dict
        All arguments. Here only the following keys are of interest:
            nnodes: int
                Number of nodes
            cpus: int
                Number of CPUs
            sinfo_pd : pd.DataFrame
                Per-node sinfo output for an extensive set of parameters
    part_pd : pd.DataFrame
        sinfo_pd reduced to the queried partition and with expanded procs info

    Returns
    -------
    directive : str
        Slurm directive to allocate specific nodes
    """
    nnodes, ncpus = int(args['nnodes']), int(args['cpus'])  # node/cpu resquest
    procs_wanted = nnodes * ncpus  # total number of processors wanted
    procs_alloc, nodes_alloc = 0, []

    # parse sinfo sorted from nodes with the most to the least avail cpus
    for row in part_pd.values:
        node = row[0]
        cpu_load = float(row[3])
        mem_load = float(row[-2])
        idle_procs = int(row[-1])
        # skip nodes that are too heavily loaded
        if cpu_load < 50 and mem_load < 50:
            procs_alloc += idle_procs
            nodes_alloc.append(node)
        # stop when enough nodes to accommodate all cpus are found
        if procs_alloc >= procs_wanted:
            break

    directive = '#SBATCH --nodelist=%s' % ','.join(nodes_alloc)
    directive += '\n#SBATCH --tasks=%s' % args['cpus']
    return directive



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import unittest
from Xhpc.nodes import *


class TestGetNodesPpn(unittest.TestCase):

    def setUp(self) -> None:
        self.args = {'torque': False, 'nnodes': 1, 'cpus': 1}

    def test_get_nodes_ppn(self):
        self.args['torque'] = True
        obs = get_nodes_ppn(self.args)
        exp = '#PBS -l nodes=1:ppn=1'
        self.assertEqual(exp, obs)

        self.args['torque'] = False
        obs = get_nodes_ppn(self.args)
        exp = '#SBATCH --ntasks=1'
        self.assertEqual(exp, obs)

        self.args['cpus'] = 2
        obs = get_nodes_ppn(self.args)
        exp = '#SBATCH --ntasks=2'
        self.assertEqual(exp, obs)

        self.args['nnodes'] = 2
        obs = get_nodes_ppn(self.args)
        exp = '#SBATCH --nodes=2\n#SBATCH --ntasks-per-node=2'
        self.assertEqual(exp, obs)

        self.args['cpus'] = 1
        obs = get_nodes_ppn(self.args)
        exp = '#SBATCH --nodes=2\n#SBATCH --ntasks-per-node=1'
        self.assertEqual(exp, obs)


class TestGetNodelist(unittest.TestCase):

    def test_get_nodelist(self):
        args = {'torque': False, 'nodes': ('a', 'b',), 'cpus': 1}

        obs = get_nodelist(args)
        exp = '#SBATCH --nodelist=a,b\n#SBATCH --ntasks-per-node=1'
        self.assertEqual(exp, obs)

        args['torque'] = True
        obs = get_nodelist(args)
        exp = '#PBS -l nodes=a,b\n#PBS -l ppn=1'
        self.assertEqual(exp, obs)


# class TestAllocateNodes(unittest.TestCase):
#
#     def setUp(self) -> None:
#         pass
#
#     def test_allocate_nodes(self):
#         allocate_nodes()
#
#     def tearDown(self) -> None:
#         pass


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2020, Franck Lejzerowicz.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import unittest
from Xhpc.preamble import *


class TestAddEnv(unittest.TestCase):

    def setUp(self):
        self.env = 'conda_env'
        self.args = {'env': None}
        self.preamble_cmd = ['\n# general environment info / behaviour',
                             'uname -a', 'set -e', 'set -u']
        self.env_cmd = ['\n# active conda environment',
                        'echo "Conda environment is %s"' % self.env,
                        'source activate %s' % self.env]

    def test_add_env_none(self):
        add_env(self.args)
        exp = self.preamble_cmd
        self.assertEqual(self.args['preamble'], exp)

    def test_add_env_empty(self):
        self.assertIsNone(self.args.get('preamble'))

    def test_add_env_return(self):
        self.assertIsNone(add_env(self.args))

    def test_add_env(self):
        self.args['env'] = self.env
        add_env(self.args)
        exp = self.preamble_cmd + self.env_cmd
        self.assertEqual(self.args['preamble'], exp)


class TestAddWorkdir(unittest.TestCase):

    def setUp(self):
        self.args = {'workdir': None, 'torque': False, 'preamble': []}
        self.workdir = '/any/path'
        self.torque = "export PBS_O_WORKDIR='%s'" % self.workdir
        self.slurm = "export SLURM_SUBMIT_DIR='%s'" % self.workdir
        self.torque_cmd = ['\n# change working directory', self.torque]
        self.slurm_cmd = ['\n# change working directory', self.slurm]

    def test_add_workdir_empty(self):
        self.assertIsNone(add_workdir(self.args))

    def test_add_workdir_slurm(self):
        self.args['workdir'] = self.workdir
        add_workdir(self.args)
        self.assertEqual(self.args['preamble'], self.slurm_cmd)

    def test_add_workdir_torque(self):
        self.args['workdir'] = self.workdir
        self.args['torque'] = True
        add_workdir(self.args)
        self.assertEqual(self.args['preamble'], self.torque_cmd)


class TestAddTmpdir(unittest.TestCase):

    def setUp(self):
        os.environ = {}
        self.args = {
            'tmp': False, 'move': False, 'torque': False,
            'job': '{A}', 'preamble': []}
        self.tmpdir_passed = '/any/path'
        self.tmpdir_move = '${SCRATCH_DIR}/tmpdir'
        self.tmpdir_os = '${TMPDIR}'

    def test_add_tmpdir_none(self):
        self.assertFalse(add_tmpdir(self.args))

    def test_add_tmpdir_tmp_slurm(self):
        self.args['tmp'] = self.tmpdir_passed
        self.tmpdir_passed += '/' + self.args['job'] + '_${SLURM_JOB_ID}'
        exp = [
            '\n# create and export the temporary directory',
            'mkdir -p %s' % self.tmpdir_passed,
            'export TMPDIR="%s"' % self.tmpdir_passed]
        add_tmpdir(self.args)
        self.assertEqual(exp, self.args['preamble'])

    def test_add_tmpdir_tmp_torque(self):
        self.args['tmp'] = self.tmpdir_passed
        self.args['torque'] = True
        self.tmpdir_passed += '/' + self.args['job'] + '_${PBS_JOBID}'
        exp = [
            '\n# create and export the temporary directory',
            'mkdir -p %s' % self.tmpdir_passed,
            'export TMPDIR="%s"' % self.tmpdir_passed]
        add_tmpdir(self.args)
        self.assertEqual(exp, self.args['preamble'])

    def test_add_tmpdir_move_slurm(self):
        self.args['tmp'] = self.tmpdir_move
        self.tmpdir_move += '/' + self.args['job'] + '_${SLURM_JOB_ID}'
        exp = [
            '\n# create and export the temporary directory',
            'mkdir -p %s' % self.tmpdir_move,
            'export TMPDIR="%s"' % self.tmpdir_move]
        add_tmpdir(self.args)
        self.assertEqual(exp, self.args['preamble'])

    def test_add_tmpdir_move_torque(self):
        self.args['tmp'] = self.tmpdir_move
        self.args['torque'] = True
        self.tmpdir_move += '/' + self.args['job'] + '_${PBS_JOBID}'
        exp = [
            '\n# create and export the temporary directory',
            'mkdir -p %s' % self.tmpdir_move,
            'export TMPDIR="%s"' % self.tmpdir_move]
        add_tmpdir(self.args)
        self.assertEqual(exp, self.args['preamble'])

    def test_add_tmpdir_os_slurm(self):
        os.environ['TMPDIR'] = 1
        self.tmpdir_os += '/' + self.args['job'] + '_${SLURM_JOB_ID}'
        exp = [
            '\n# create and export the temporary directory',
            'mkdir -p %s' % self.tmpdir_os,
            'export TMPDIR="%s"' % self.tmpdir_os]
        add_tmpdir(self.args)
        self.assertEqual(exp, self.args['preamble'])

    def test_add_tmpdir_os_torque(self):
        os.environ['TMPDIR'] = 1
        self.args['torque'] = True
        self.tmpdir_os += '/' + self.args['job'] + '_${PBS_JOBID}'
        exp = [
            '\n# create and export the temporary directory',
            'mkdir -p %s' % self.tmpdir_os,
            'export TMPDIR="%s"' % self.tmpdir_os]
        add_tmpdir(self.args)
        self.assertEqual(exp, self.args['preamble'])


class TestAddProcsNodes(unittest.TestCase):

    def setUp(self):
        self.args = {'torque': False, 'preamble': []}
        self.slurm = '$SLURM_JOB_NODELIST'
        self.torque = '$PBS_NODEFILE'

    def test_add_procs_nodes_slurm(self):
        add_procs_nodes(self.args)
        exp = ['NPROCS=`wc -l < %s`' % self.slurm,
               'NNODES=`uniq %s | wc -l`' % self.slurm]
        self.assertEqual(exp, self.args['preamble'])

    def test_add_procs_nodes_torque(self):
        self.args['torque'] = True
        add_procs_nodes(self.args)
        exp = ['NPROCS=`wc -l < %s`' % self.torque,
               'NNODES=`uniq %s | wc -l`' % self.torque]
        self.assertEqual(exp, self.args['preamble'])


class TestAddEchoes(unittest.TestCase):

    def setUp(self):
        self.args = {'torque': False, 'preamble': [], 'std_path': 'X'}
        self.part1 = ['\n# echo some info about the job',
                      'echo Running on host `hostname`',
                      'echo Time is `date`', 'echo Directory is `pwd`']
        self.slurm = ['NPROCS=`wc -l < $SLURM_JOB_NODELIST`',
                      'NNODES=`uniq $SLURM_JOB_NODELIST | wc -l`']
        self.torque = ['NPROCS=`wc -l < $PBS_NODEFILE`',
                       'NNODES=`uniq $PBS_NODEFILE | wc -l`']
        self.part2 = ['echo Use ${NPROCS} procs on ${NNODES} nodes',
                      'echo Job stdout is X.o', 'echo Job stderr is X.e']

    def test_add_echoes_notmp_slurm(self):
        exp = self.part1 + self.slurm + self.part2
        add_echoes(self.args)
        self.assertEqual(self.args['preamble'], exp)

    def test_add_echoes_notmp_torque(self):
        self.args['torque'] = True
        exp = self.part1 + self.torque + self.part2
        add_echoes(self.args)
        self.assertEqual(self.args['preamble'], exp)


if __name__ == '__main__':
    unittest.main()



# ----------------------------------------------------------------------------
# Copyright (c) 2022, Franck Lejzerowicz.
#
# Distributed under the terms of the MIT License.
#
# The full license is in the file LICENSE, distributed with this software.
# ----------------------------------------------------------------------------

import re
import ast
from setuptools import find_packages, setup

classes = """
    License :: OSI Approved :: BSD License
    Topic :: Scientific/Engineering
    Topic :: Scientific/Engineering :: Bio-Informatics
    Programming Language :: Python :: 3.5
    Programming Language :: Python :: 3 :: Only
    Operating System :: Unix
    Operating System :: POSIX
    Operating System :: MacOS :: MacOS X
"""

classifiers = [s.strip() for s in classes.split('\n') if s]

description = (
    "Xhpc is a command line tool allowing the conversion of a sh/bash script "
    "into a Slurm (or Torque) script with directives for use on a computer"
    "cluster."
)

with open("README.md") as f:
    long_description = f.read()

_version_re = re.compile(r"__version__\s+=\s+(.*)")

with open("Xhpc/__init__.py", "rb") as f:
    hit = _version_re.search(f.read().decode("utf-8")).group(1)
    version = str(ast.literal_eval(hit))

standalone = ['Xhpc=Xhpc.scripts._standalone_xhpc:standalone_xhpc']

setup(
    name="Xhpc",
    version=version,
    license="BSD",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Franck Lejzerowicz",
    author_email="franck.lejzerowicz@gmail.com",
    maintainer="Franck Lejzerowicz",
    maintainer_email="franck.lejzerowicz@gmail.com",
    url="https://github.com/FranckLejzerowicz/Xhpc",
    packages=find_packages(),
    install_requires=["click", "pandas", "PyYAML"],
    classifiers=classifiers,
    entry_points={'console_scripts': standalone},
    python_requires='>=3.6',
    package_data={'Xhpc': ['config/config.txt']}
)
